<?php

namespace SpectraPro\BlocksConfig\FreemiumBlocks\Slider;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Class Block to handle filter to extend slider.
 */
class Block {

	/**
	 * Init
	 */
	public static function init() {

		$self = new self();
		add_filter( 'uagb_slider_options', array( $self, 'extend_slider_options' ), 10, 2 );
		add_action( 'spectra_after_slider_options_loaded', array( $self, 'register_nav_events' ) );
		add_filter( 'spectra_slider_styling', array( $self, 'extend_slider_styling' ), 10, 2 );
		add_filter( 'spectra_frontend_static_style', array( $self, 'load_slider_pro_styles' ), 10, 2 );
		add_filter( 'spectra_slider_frontend_attributes', array( $self, 'extend_frontend_attributes' ) );
	}

	/**
	 * Extend slider options.
	 *
	 * @param array $options slider options.
	 * @param array $attributes block attributes.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function extend_slider_options( $options, $attributes ) {

		if ( isset( $attributes['sliderStyle'] ) && 'focus' === $attributes['sliderStyle'] ) {

			$slides_width = (int) $attributes['consecutiveSlidesWidth'];

			$options['slidesPerView']       = 1 + ( $slides_width / 100 );
			$options['centeredSlides']      = true;
			$options['spaceBetween']        = 10;
			$options['slideToClickedSlide'] = true;
		}

		if ( isset( $attributes['enableHashNavigation'] ) && $attributes['enableHashNavigation'] ) {
			$options['hashNavigation'] = array(
				'replaceState' => true,
				'watchState'   => true,
			);
		}

		if ( ! empty( $attributes['sliderStyle'] ) && 'normal' === $attributes['sliderStyle'] && ! empty( $attributes['slidesPerView'] ) ) {
			$slides_desktop = (float) $attributes['slidesPerView'];
			$slides_tablet  = ! empty( $attributes['slidesPerViewTablet'] ) ? (float) $attributes['slidesPerViewTablet'] : $slides_desktop;
			$slides_mobile  = ! empty( $attributes['slidesPerViewMobile'] ) ? (float) $attributes['slidesPerViewMobile'] : $slides_tablet;

			$options['breakpoints'] = array(
				UAGB_TABLET_BREAKPOINT => array(
					'slidesPerView' => $slides_desktop,
				),
				// when window width is >= 767px.
				UAGB_MOBILE_BREAKPOINT => array(
					'slidesPerView' => $slides_tablet,
				),
				// when window width is >= 320px.
				320                    => array(
					'slidesPerView' => $slides_mobile,
				),
			);
		}

		return $options;

	}

	/**
	 * Extend slider style.
	 *
	 * @param array $selectors slider selectors.
	 * @param array $attributes block attributes.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function extend_slider_styling( $selectors, $attributes ) {

		if ( isset( $attributes['sliderStyle'] ) && 'focus' === $attributes['sliderStyle'] ) {

			// If the transition speed is set, use half of it - else use the default.
			$fade_duration = is_numeric( $attributes['transitionSpeed'] )
				? $attributes['transitionSpeed'] / 2
				: 0;

			$selectors[ '.uagb-block-' . $attributes['block_id'] . ' .swiper-slide::before' ]['transition-duration']                            = $fade_duration . 'ms';
			$selectors[ '.uagb-block-' . $attributes['block_id'] . ' .uagb-slider-child-wrap:not(.swiper-slide-active)::before' ]['background'] = $attributes['slideOverlayColor'];

		}

		return $selectors;

	}

	/**
	 * Load slider pro static style on front end.
	 *
	 * @param string $style slider style.
	 * @param string $block_name block name.
	 *
	 * @since 1.0.0
	 * @return string
	 */
	public function load_slider_pro_styles( $style, $block_name ) {

		if ( 'slider' === $block_name ) {

			$block_static_css_path = SPECTRA_PRO_DIR . 'assets/css/blocks/slider-pro.css';

			if ( file_exists( $block_static_css_path ) ) {

				$file_system = uagb_filesystem();

				$style .= $file_system->get_contents( $block_static_css_path );
			}
		}

		return $style;
	}

	/**
	 * Extend slider front end attributes to work default values.
	 *
	 * @param array $slider_attributes slider attributes.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function extend_frontend_attributes( $slider_attributes ) {

		return array_merge(
			array(
				'enableHashNavigation'         => false,
				'navigationLink'               => '',
				'consecutiveSlidesWidth'       => 35,
				'consecutiveSlidesWidthTablet' => 25,
				'consecutiveSlidesWidthMobile' => 30,
				'sliderStyle'                  => 'normal',
				'slideOverlayColor'            => 'rgba(0,0,0,.5)',
				'slidesPerView'                => 1,
				'slidesPerViewTablet'          => '',
				'slidesPerViewMobile'          => '',
			),
			$slider_attributes
		);

	}

	/**
	 * Register click events for custom navigation.
	 *
	 * @param array $attributes Array of block attributes.
	 * @since 1.0.0
	 */
	public function register_nav_events( $attributes ) {

		if ( empty( $attributes['enableCustomNavigation'] ) ) {
			return;
		}

		$selector  = '.uagb-block-' . $attributes['block_id'] . ' .uagb-swiper';
		$slider_id = isset( $attributes['sliderID'] ) ? $attributes['sliderID'] : '';
		?>

		window.addEventListener("DOMContentLoaded", function(){

			const swiper = document.querySelector("<?php echo esc_attr( $selector ); ?>").swiper;

			const slideNextElements = document.querySelectorAll('.slider-<?php echo esc_attr( $slider_id ); ?>-next');

			slideNextElements.forEach(element => {
				element.addEventListener('click', event => {
					swiper?.slideNext();
				});
			});

			const slidePrevElements = document.querySelectorAll('.slider-<?php echo esc_attr( $slider_id ); ?>-prev');

			slidePrevElements.forEach(element => {
				element.addEventListener('click', event => {
					swiper?.slidePrev();
				});
			});
		});
		<?php
	}
}

